
/*
 * Copyright (c) 2015, Nordic Semiconductor
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.nordicsemi.nrfUARTv2;


import java.io.UnsupportedEncodingException;
import java.text.DateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.util.StringTokenizer;

import com.nordicsemi.nrfUARTv2.UartService;

import android.app.Activity;
import android.app.AlertDialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothManager;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.ServiceConnection;
import android.content.res.Configuration;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.support.v4.content.LocalBroadcastManager;
import android.util.Log;
import android.view.View;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.preference.PreferenceManager;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.RadioGroup;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.SimpleAdapter;

import android.media.AudioManager;
import android.media.ToneGenerator;
import android.os.Vibrator;


public class MainActivity extends Activity implements RadioGroup.OnCheckedChangeListener {
    private static final int REQUEST_SELECT_DEVICE = 1;
    private static final int REQUEST_ENABLE_BT = 2;
    private static final int UART_PROFILE_READY = 10;
    public static final String TAG = "nRFUART";
    private static final int UART_PROFILE_CONNECTED = 20;
    private static final int UART_PROFILE_DISCONNECTED = 21;
    private static final int STATE_OFF = 10;

    private int mState = UART_PROFILE_DISCONNECTED;
    private UartService mService = null;
    private BluetoothDevice mDevice = null;
    private BluetoothAdapter mBtAdapter = null;
    private ListView messageListView;
    private ArrayAdapter<String> listAdapter;
    private Button btnConnectDisconnect;
    private TextView textBattLevel;
    private String mSavedDevice = null;
    private List<HashMap<String,String>> DevInfoList = null;
    private ListView infoListView;
    private SimpleAdapter infoAdapter;

    private ToneGenerator toneGenerator = null;
    public static boolean autoConnectChecked;

    // バーコードデータバッファ
    private String strBarData = "";

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        mBtAdapter = BluetoothAdapter.getDefaultAdapter();
        if (mBtAdapter == null) {
            Toast.makeText(this, "Bluetooth is not available", Toast.LENGTH_LONG).show();
            finish();
            return;
        }

        messageListView = (ListView) findViewById(R.id.listMessage);
        listAdapter = new ArrayAdapter<String>(this, R.layout.message_detail);
        messageListView.setAdapter(listAdapter);
        messageListView.setDivider(null);
        btnConnectDisconnect=(Button) findViewById(R.id.btn_select);
        textBattLevel = (TextView) findViewById(R.id.battlevel);
        toneGenerator = new ToneGenerator(AudioManager.STREAM_SYSTEM, ToneGenerator.MAX_VOLUME);

        // プリファレンスから登録デバイスアドレスを取得する
        SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(MainActivity.this);
        mSavedDevice = pref.getString("DefaultDevice", "");
                     
        if (mSavedDevice == null || mSavedDevice.length() == 0) {
            autoConnectChecked = false;
        }
        else {
            autoConnectChecked = true;
        }
        service_init();

        // Handle Disconnect & Connect button
        btnConnectDisconnect.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (!mBtAdapter.isEnabled()) {
                    Log.i(TAG, "onClick - BT not enabled yet");
                    Intent enableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
                    startActivityForResult(enableIntent, REQUEST_ENABLE_BT);
                }
                else {
                	if (btnConnectDisconnect.getText().equals("受信開始")) {
                        //プリファレンスから登録デバイスアドレスを取得する
                        SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(MainActivity.this);
                        String savedDevice = pref.getString("DefaultDevice", "");
                     
                        if (savedDevice == null || savedDevice.length() == 0) {
                            //登録デバイス無しの場合はDeviceListActibityクラスを呼び出し、周辺デバイスを検索する
                            Intent newIntent = new Intent(MainActivity.this, DeviceListActivity.class);
                            startActivityForResult(newIntent, REQUEST_SELECT_DEVICE);
                        }
                        else {
                            //登録デバイス有りの場合は即接続処理を行う 
                            mDevice = BluetoothAdapter.getDefaultAdapter().getRemoteDevice(savedDevice);
                            mService.connect(savedDevice);
                        }
                    } else {
                         // Disconnect button pressed
                         if (mDevice!=null) {
                            mService.disconnect();
                        }
                    }
                }
            }
        });
        // Set initial UI state
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.main_menu, menu);
        return true;
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        super.onPrepareOptionsMenu(menu);
        menu.findItem(R.id.menuitem2).setChecked(autoConnectChecked);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch(item.getItemId()){
            case R.id.menuitem1:
                // 接続先デバイス検索処理
                Intent newIntent = new Intent(MainActivity.this, DeviceListActivity.class);
                startActivityForResult(newIntent, REQUEST_SELECT_DEVICE);
                return true;

            case R.id.menuitem2:
                // 自動再接続の設定
                if ((item.isChecked() == false) && (mState != UART_PROFILE_CONNECTED)) {
                    showMessage("デバイスと接続されていません");
                    return true;
                }
                SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(MainActivity.this);
                Editor editor = pref.edit();

                // チェックボックスの状態変更を行う
                item.setChecked(!item.isChecked());
                // 反映後の状態を取得する
                autoConnectChecked = item.isChecked();
                if (autoConnectChecked == true) {
                    //プリファレンスへBDアドレス保存
                    editor.putString("DefaultDevice", mDevice.getAddress());
                    mSavedDevice = mDevice.getAddress(); 
               }
                else {
                    //プリファレンスのクリア
                    editor.putString("DefaultDevice", "");
                }
                editor.commit();
                return true;

            case R.id.menuitem3:
                // デバイス情報(DIS)の表示
                if (mState != UART_PROFILE_CONNECTED) {
                    showMessage("デバイスと接続されていません");
                    return true;
                }
                // 接続後最初の1回のみ取得処理を行う
                if (DevInfoList == null) { 
                    // DISキャラクタリスティックの取得
                     mService.getDeviceInformation();
                    // DIS項目リストの作成
                    DevInfoList = new ArrayList<HashMap<String,String>>();
                }
                // リストビューの作成
                infoListView = new ListView(this); 

                infoAdapter = new SimpleAdapter(
                    this,
                    DevInfoList,
                    R.layout.info_item,
                    new String[] { "title", "content" },
                    new int[] { R.id.title, R.id.content }
                );
                infoListView.setAdapter(infoAdapter);

                // アラートダイアログの表示
                AlertDialog.Builder disDialog = new AlertDialog.Builder(MainActivity.this); 

                disDialog.setTitle("デバイス情報の表示");
                disDialog.setView(infoListView);

                disDialog.setPositiveButton("OK", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        dialog.dismiss();
                    }
                });
                disDialog.create();
                disDialog.show();                
                
                return true;
        }
        return false;
    }

    private void addDisItemList(String title, String content) {
        HashMap<String,String> map = new HashMap<String,String>();
        
        // タイトルと取得値をセット
        map.put("title", title);
        map.put("content", content);

        // リストビューに追加
        DevInfoList.add(map);
        // データの更新を反映
        infoAdapter.notifyDataSetChanged();
    }

    //UART service connected/disconnected
    private ServiceConnection mServiceConnection = new ServiceConnection() {
        public void onServiceConnected(ComponentName className, IBinder rawBinder) {
                mService = ((UartService.LocalBinder) rawBinder).getService();
                Log.d(TAG, "onServiceConnected mService= " + mService);
                if (!mService.initialize()) {
                    Log.e(TAG, "Unable to initialize Bluetooth");
                    finish();
                }
        }

        public void onServiceDisconnected(ComponentName classname) {
       ////     mService.disconnect(mDevice);
        		mService = null;
        }
    };

    private Handler mHandler = new Handler() {
        @Override
        
        // Handler events that received from UART service 
        public void handleMessage(Message msg) {
  
        }
    };

    private final BroadcastReceiver UARTStatusChangeReceiver = new BroadcastReceiver() {

        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();

            final Intent mIntent = intent;
            // 接続検知イベント
            if (action.equals(UartService.ACTION_GATT_CONNECTED)) {
            	 runOnUiThread(new Runnable() {
                     public void run() {
                             showMessage("デバイスと接続されました");
                             String currentDateTimeString = DateFormat.getTimeInstance().format(new Date());
                             Log.d(TAG, "UART_CONNECT_MSG");
                             btnConnectDisconnect.setText("受信停止");
                             ((TextView) findViewById(R.id.deviceName)).setText(mDevice.getName());
                             //listAdapter.add("["+currentDateTimeString+"] Connected to: "+ mDevice.getName());
                        	 //messageListView.smoothScrollToPosition(listAdapter.getCount() - 1);
                             DevInfoList = null;
                             mState = UART_PROFILE_CONNECTED;
                     }
            	 });
            }
           
            // 切断検知イベント
            if (action.equals(UartService.ACTION_GATT_DISCONNECTED)) {
            	 runOnUiThread(new Runnable() {
                     public void run() {
                             showMessage("デバイスと切断されました");
                             String currentDateTimeString = DateFormat.getTimeInstance().format(new Date());
                             Log.d(TAG, "UART_DISCONNECT_MSG");
                             btnConnectDisconnect.setText("受信開始");
                             ((TextView) findViewById(R.id.deviceName)).setText("未接続");
                             textBattLevel.setText("");
                             //listAdapter.add("["+currentDateTimeString+"] Disconnected to: "+ mDevice.getName());
                             mState = UART_PROFILE_DISCONNECTED;
                             mService.close();
                             //setUiState();
                         
                     }
                 });
            }

            // GATTサービス検出イベント
            if (action.equals(UartService.ACTION_GATT_SERVICES_DISCOVERED)) {
             	 mService.enableTXNotification();
                 mService.enableBatteryNotification();
            }

            // データ受信イベント
            if (action.equals(UartService.ACTION_DATA_AVAILABLE)) {
              
                final byte[] txValue = intent.getByteArrayExtra(UartService.EXTRA_DATA);
                // 分割パケットの総数/分割パケットインデックス値
                final int packetCnt = (int)txValue[0];
                final int packetIdx = (int)txValue[1];
                
                runOnUiThread(new Runnable() {
                    public void run() {
                         try {
                            // 読み取りデータ部を抽出
                            String text = new String(txValue, "UTF-8");
                            strBarData += text.substring(2, text.length());

                            // 時間データを取得(このプログラムでは非表示) 
                            String currentDateTimeString = DateFormat.getTimeInstance().format(new Date());

                            // 最終パケットなら画面にデータを出力
                            if (packetIdx == packetCnt) { 
                                // ブザーとバイブレータを鳴動。
                                toneGenerator.startTone(ToneGenerator.TONE_PROP_BEEP);
                                Vibrator vibrator = (Vibrator) getSystemService(VIBRATOR_SERVICE);
                                vibrator.vibrate(100);

                                listAdapter.add(strBarData);
                                messageListView.smoothScrollToPosition(listAdapter.getCount() - 1);

                                // バーコードバッファのクリア
                                strBarData = "";
                            }
                         } catch (Exception e) {
                             Log.e(TAG, e.toString());
                         }
                     }
                });
             }

             // バッテリレベル受信イベント
             if (action.equals(UartService.ACTION_BATT_AVAILABLE)) {
              
                 final byte[] lvValue = intent.getByteArrayExtra(UartService.EXTRA_LEVEL);
                 runOnUiThread(new Runnable() {
                     public void run() {
                         try {
                             // 画面左下に "XX/100%" の形式で表示
                             String s = String.format("%d/100%%", (int)lvValue[0]);
                             textBattLevel.setText(s);
                             
                         } catch (Exception e) {
                             Log.e(TAG, e.toString());
                         }
                     }
                 });
            }

            // デバイス情報受信イベント
            if (action.equals(UartService.ACTION_INFO_AVAILABLE)) {

                final String text = intent.getStringExtra(UartService.EXTRA_INFO);
                    runOnUiThread(new Runnable() {
                        public void run() {
                        try {
                            StringTokenizer st = new StringTokenizer(text, ":");  

                            // リストビューを更新
                            String title = st.nextToken();
                            String content = st.nextToken();
                            addDisItemList(title, content);
                       
                        } catch (Exception e) {
                             Log.e(TAG, e.toString());
                        }
                    }
                });
             }
           //*********************//
            if (action.equals(UartService.DEVICE_DOES_NOT_SUPPORT_UART)){
            	showMessage("Device doesn't support UART. Disconnecting");
            	mService.disconnect();
            }
 
        }
    };

    private void service_init() {
        Intent bindIntent = new Intent(this, UartService.class);
        bindService(bindIntent, mServiceConnection, Context.BIND_AUTO_CREATE);
  
        LocalBroadcastManager.getInstance(this).registerReceiver(UARTStatusChangeReceiver, makeGattUpdateIntentFilter());
    }

    private static IntentFilter makeGattUpdateIntentFilter() {
        final IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(UartService.ACTION_GATT_CONNECTED);
        intentFilter.addAction(UartService.ACTION_GATT_DISCONNECTED);
        intentFilter.addAction(UartService.ACTION_GATT_SERVICES_DISCOVERED);
        intentFilter.addAction(UartService.ACTION_DATA_AVAILABLE);
        intentFilter.addAction(UartService.ACTION_BATT_AVAILABLE);
        intentFilter.addAction(UartService.ACTION_INFO_AVAILABLE);
        intentFilter.addAction(UartService.DEVICE_DOES_NOT_SUPPORT_UART);
        return intentFilter;
    }

    @Override
    public void onStart() {
        super.onStart();
    }

    @Override
    public void onDestroy() {
    	 super.onDestroy();
        Log.d(TAG, "onDestroy()");
        
        try {
        	LocalBroadcastManager.getInstance(this).unregisterReceiver(UARTStatusChangeReceiver);
        } catch (Exception ignore) {
            Log.e(TAG, ignore.toString());
        } 
        unbindService(mServiceConnection);
        mService.stopSelf();
        mService= null;
    }

    @Override
    protected void onStop() {
        Log.d(TAG, "onStop");
        super.onStop();
    }

    @Override
    protected void onPause() {
        Log.d(TAG, "onPause");
        super.onPause();
    }

    @Override
    protected void onRestart() {
        super.onRestart();
        Log.d(TAG, "onRestart");
    }

    @Override
    public void onResume() {
        super.onResume();
        Log.d(TAG, "onResume");
        if (!mBtAdapter.isEnabled()) {
            Log.i(TAG, "onResume - BT not enabled yet");
            Intent enableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startActivityForResult(enableIntent, REQUEST_ENABLE_BT);
        }
    }

    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        switch (requestCode) {

        case REQUEST_SELECT_DEVICE:
        	//When the DeviceListActivity return, with the selected device address
            if (resultCode == Activity.RESULT_OK && data != null) {
                String deviceAddress = data.getStringExtra(BluetoothDevice.EXTRA_DEVICE);
                mDevice = BluetoothAdapter.getDefaultAdapter().getRemoteDevice(deviceAddress);
               
                Log.d(TAG, "... onActivityResultdevice.address==" + mDevice + "mserviceValue" + mService);
                ((TextView) findViewById(R.id.deviceName)).setText(mDevice.getName()+ " - connecting");
                mService.connect(deviceAddress);
             }
            break;
        case REQUEST_ENABLE_BT:
            // When the request to enable Bluetooth returns
            if (resultCode == Activity.RESULT_OK) {
                Toast.makeText(this, "Bluetooth has turned on ", Toast.LENGTH_SHORT).show();

            } else {
                // User did not enable Bluetooth or an error occurred
                Log.d(TAG, "BT not enabled");
                Toast.makeText(this, "Problem in BT Turning ON ", Toast.LENGTH_SHORT).show();
                finish();
            }
            break;
        default:
            Log.e(TAG, "wrong request code");
            break;
        }
    }

    @Override
    public void onCheckedChanged(RadioGroup group, int checkedId) {
       
    }

    private void showMessage(String msg) {
        Toast.makeText(this, msg, Toast.LENGTH_SHORT).show();
    }

    @Override
    public void onBackPressed() {
        if (mState == UART_PROFILE_CONNECTED) {
            Intent startMain = new Intent(Intent.ACTION_MAIN);
            startMain.addCategory(Intent.CATEGORY_HOME);
            startMain.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            startActivity(startMain);
            showMessage("WRS-Sample's running in background.\n             Disconnect to exit");
        }
        else {
            new AlertDialog.Builder(this)
            .setIcon(android.R.drawable.ic_dialog_alert)
            .setTitle(R.string.popup_title)
            .setMessage(R.string.popup_message)
            .setPositiveButton(R.string.popup_yes, new DialogInterface.OnClickListener()
                {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
   	                finish();
                }
            })
            .setNegativeButton(R.string.popup_no, null)
            .show();
        }
    }
}
